/*
 * Copyright (C) 2020 Tianjin KYLIN Information Technology Co., Ltd.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, see <http://www.gnu.org/licenses/>.
 *
**/
#include <QDBusInterface>
#include <QDBusReply>
#include "msysdbus.h"
#include "rsac.h"
#include "lockdialogmodel.h"
#include <openssl/rsa.h>
#include <openssl/pem.h>
#include <QTimer>

MSysDbus::MSysDbus(LockDialogModel *model, QObject *parent)
    : QObject(parent)
    , m_modelLockDialog(model) 

{
    registerUniauth();
}

bool MSysDbus::sendPassword(const QString username,QByteArray password)
{

    if(m_modelLockDialog){
        return Q_EMIT m_modelLockDialog->sendPassword(username,password);
    }
    return true;
}

QString MSysDbus::getPublicEncrypt()
{
    if(m_modelLockDialog){
        return Q_EMIT m_modelLockDialog->getPublicEncrypt();
    }
    return "";
}

void MSysDbus::initRSA()
{
    //rsac.generateKeyPair(priKey, pubKey, 2048);
}

void MSysDbus::registerUniauth()
{
    QDBusInterface iface("org.ukui.UniauthBackend",
                         "/org/ukui/UniauthBackend",
                         "org.ukui.UniauthBackend",
                         QDBusConnection::systemBus());
    QDBusReply<QString> stateReply = iface.call("registerLoginApp");
    if(!stateReply.isValid())
    {
        qDebug() << "registerLoginApp:" << stateReply.error();
        return ;
    }
}


void MSysDbus::sendSignalLoginFinished(QString username,bool res)
{
    QDBusInterface iface("org.ukui.UniauthBackend",
                         "/org/ukui/UniauthBackend",
                         "org.ukui.UniauthBackend",
                         QDBusConnection::systemBus());
    QDBusReply<QString> stateReply = iface.call("sendSignalLoginFinished",username,res);
}

